# （略）
from urllib.parse import urljoin
from bs4 import BeautifulSoup
# （略）

def scrape_list_page(response: requests.Response) -> Iterator[str]:
    """
    一覧ページのResponseから詳細ページのURLを抜き出すジェネレーター関数。
    """
    soup = BeautifulSoup(response.text, 'html.parser')

    for a in soup.select('#listBook > li > a[itemprop="url"]'):
        url = urljoin(response.url, a.get('href'))  # 絶対URLに変換する。
        yield url


def scrape_detail_page(response: requests.Response) -> dict:
    """
    詳細ページのResponseから電子書籍の情報をdictで取得する。
    """
    soup = BeautifulSoup(response.text, 'html.parser')
    ebook = {
        'url': response.url,  # URL
        'key': extract_key(response.url),  # URLから抜き出したキー
        'title': soup.select_one('#bookTitle').text,  # タイトル。select_one()はselect()の結果の最初の要素を取得するメソッド。
        'price': soup.select_one('.buy').contents[0].strip(),  # 価格（strip()で前後の空白を削除）
        'content': [normalize_spaces(h3.text) for h3 in soup.select('#content > h3')],  # 目次
    }
    return ebook  # dictを返す。
